﻿// *                                                               *
// *       Copyright (c) Fast Corporation, 2007                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

// -------------------------------------------------------------------
// FVX Sample Program
// 
//
// 作成者 : R.Nagashima
// 履歴   : 2007/02/06 初版
//
//-------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
// FVXデータ構造の使用宣言
using FVDATASLib;


namespace BinFilter
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }
        // ビデオデバイスがオープンされているか
        private bool m_blnVideoOpened = false;
        // ビデオ設定ファイル名（フルパス）
        private string m_strVideoConfFilePath = "";
        // 初期ディレクトリ
        private string m_strDirectory;
        //処理範囲
        private FvRectangle m_ExeRegion = new FvRectangle();
		//2値しきい値
		private int m_BinLevel = 128;


        private void frmMain_Load(object sender, EventArgs e)
        {
            int IntErr;
            frmSelectvideoConfFile fVideoConf = new frmSelectvideoConfFile();
            frmLicense fLicense = new frmLicense();

            // 変数初期化
            m_blnVideoOpened = false;
            btnGrab.Enabled = false;

            cmbDirection1.Items.Insert(0, "垂直方向");//Y_DIRECTION
            cmbDirection1.Items.Insert(1, "水平方向");//X_DIRECTION
            cmbDirection1.Items.Insert(2, "上方向");//TOP_DIRECTION
            cmbDirection1.Items.Insert(3, "下方向");//DOWN_DIRECTION
            cmbDirection1.Items.Insert(4, "左方向");//LEFT_DIRECTION
            cmbDirection1.Items.Insert(5, "右方向");//RIGHT_DIRECTION
            cmbDirection1.SelectedIndex = 0;

            cmbDirection2.Items.Insert(0, "YX_DIRECTION");
            cmbDirection2.Items.Insert(1, "XY_DIRECTION");
            cmbDirection2.Items.Insert(2, "Y_ONLY_DIRECTION");
            cmbDirection2.Items.Insert(3, "X_ONLY_DIRECTION");
            cmbDirection2.SelectedIndex = 0;

            //画面の生成
            axFvxDisplayEx1.CreateDisplay();

            //処理範囲の初期値設定（画像の全面に設定）
            m_ExeRegion.xs = -1;
            m_ExeRegion.xe = -1;
            m_ExeRegion.ys = -1;
            m_ExeRegion.ye = -1;

            //作業ディレクトリ退避
            m_strDirectory = Directory.GetCurrentDirectory();

            //使用許諾の確認
            if (fLicense.ShowDialog() == DialogResult.OK)
            {
                //ビデオ設定ファイルの選択
                if (fVideoConf.ShowDialog() == DialogResult.OK)
                    m_strVideoConfFilePath = fVideoConf.strVideoConfFilePath;

                if (fVideoConf.chkNotUseCamera.Checked == false)
                {
                    //ビデオ設定ファイルをコンポーネントに設定
                    axFvxVideo1.VideoConfFile = m_strVideoConfFilePath;

                    //デバイスオープン
                    try
                    {
                        axFvxVideo1.OpenDevice();
                    }
                    catch (Exception)
                    {
                        axFvxVideo1.GetErrReport(out IntErr);
                        MessageBox.Show("ビデオデバイスのオープンに失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
                        return;
                    }

                    //VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
                    //このため、弊社キャプチャボードFVC02では350msのセットアップタイムが必要になります｡
                    //本プログラムでは簡略化のために、一律に350msのセットアップタイムを確保していますが、
                    //お使いのキャプチャボードに合わせて変更してください。
                    Thread.Sleep(350);

                    //GrabSync/GrabASyncメソッドではImegeサイズの変更を行いませんので、
                    //あらかじめChangeImageSizeメソッドにてImageのサイズを変更してください
                    //（取り込みサイズとImageのサイズが違う場合エラーになります）
                    axFvxVideo1.ChangeImageSize(axFvxImageOrg.GetOcx());
                    try
                    {
                        // 同期取込
                        axFvxVideo1.GrabSync(axFvxImageOrg.GetOcx());
                    }
                    catch (Exception)
                    {
                        axFvxVideo1.GetErrReport(out IntErr);
                        MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
                        return;
                    }

                    //ビデオデバイスの状態を保存
                    m_blnVideoOpened = true;
                    btnGrab.Enabled = true;

                    //画像を2値化＆表示
					Binarize();
                }
            }
            else
            {
                //終了
                Application.Exit();
                return;
            }

        }


        //終了時
        private void frmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (m_blnVideoOpened)
                axFvxVideo1.CloseDevice();
        }

        //カメラ取込
        private void btnGrab_Click(object sender, EventArgs e)
        {
            //画像ファイル読み込みによって、画像サイズが初期化時から
            //変更されている可能性があるため、ChangeImageSizeを行います。
            //画像サイズが変更される可能性が無い場合は、初期化時に一度だけ行えば問題有りません。
            axFvxVideo1.ChangeImageSize(axFvxImageOrg.GetOcx());

            try
            {
                //同期取込
                axFvxVideo1.GrabSync(axFvxImageOrg.GetOcx());
                //画像2値化＆表示
				if (txtBinLevel.Text.Length == 0)
				{
					txtBinLevel.Text = "128";
					m_BinLevel = 128;
				}
				Binarize();
            }
            catch (Exception)
            {
                int IntErr;
                axFvxVideo1.GetErrReport(out IntErr);
                MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString());
            }
        }

        //画像ファイル読込
        private void btnLoadImage_Click(object sender, EventArgs e)
        {
            // ダイアログにて画像ファイルを選択します｡
            openFileDialog1.InitialDirectory = m_strDirectory;
            openFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
            openFileDialog1.DefaultExt = "*.bmp";
            openFileDialog1.ShowDialog();

			if (txtBinLevel.Text.Length == 0)
			{
				txtBinLevel.Text = "128";
				m_BinLevel = 128;
			}

            if (openFileDialog1.FileName.Length > 0)
            {
                //FileAccessコンポーネントを用いて、
                //ファイルからImageに画像を読み込みます｡
                axFvxFileAccess1.FilePath = openFileDialog1.FileName;

                try
                {
                    axFvxFileAccess1.LoadBitmap(axFvxImageTemp.GetOcx());
                }
                catch (Exception)
                {
                    int IntErr = 0;
                    axFvxFileAccess1.GetErrReport(out IntErr);
                    MessageBox.Show("読込みに失敗しました｡ ErrCode= " + IntErr.ToString());
                    return;
                }

                //画像タイプチェック
                if (axFvxImageTemp.ImageType != FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray || axFvxImageTemp.ImageWidth != 512 || axFvxImageTemp.ImageHeight != 480)
                {
                    MessageBox.Show("512*480 1/8bitグレイの画像ではありません｡", "error", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    axFvxImageTemp.BitPerPixel = 8;
                    axFvxImageTemp.ImageType = FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray;
                    axFvxImageTemp.ImageWidth = 512;
                    axFvxImageTemp.ImageHeight = 480;
                }

                //2値か濃淡か
                if (axFvxImageTemp.BitPerPixel == 1)
                {
                    axFvxImageTemp.CopyTo(axFvxImageOrgBin.GetOcx());
                    axFvxDisplayEx1.DrawImage(axFvxImageOrgBin.GetOcx());
                }
                else if (axFvxImageTemp.BitPerPixel == 8)
                {
                    axFvxImageTemp.CopyTo(axFvxImageOrg.GetOcx());
                    Binarize();
                }
                else
                    MessageBox.Show("8bit以外のbmpです", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //2値化
        private void Binarize()
		{
			axFvxChangeImage1.FixedLevel(axFvxImageOrg.GetOcx(), axFvxImageOrgBin.GetOcx(), m_BinLevel);
            axFvxDisplayEx1.DrawImage(axFvxImageOrgBin.GetOcx());
        }

        //処理範囲設定
        private void btnExeRegion_Click(object sender, EventArgs e)
        {
            int IntOK = 0;
            double dblTemp = 0.0;

            axFvxROI1.SetRectangle(m_ExeRegion);

            // FvxROIを使って処理範囲選択ダイアログを出します｡
            axFvxROI1.UsePoint = false;
            axFvxROI1.MakeROIDialog(axFvxImageOrgBin.GetOcx(), out IntOK);

            if ((IntOK == 1) && (axFvxROI1.CheckRectangle == 1))
            {
                axFvxROI1.GetRectangle(m_ExeRegion);
                // 始点と終点が逆になっていないかチェックします｡
                // FvxROIは矩形データで、左上・右下の座標位置(xs > xe, ys > ye)が
                // 逆になっていた場合でも、入れ替えずそのまま結果を返しますので、注意してください
                if (m_ExeRegion.xe < m_ExeRegion.xs)
                {
                    dblTemp = m_ExeRegion.xs;
                    m_ExeRegion.xs = m_ExeRegion.xe;
                    m_ExeRegion.xe = dblTemp;
                }
                if (m_ExeRegion.ye < m_ExeRegion.ys)
                {
                    dblTemp = m_ExeRegion.ys;
                    m_ExeRegion.ys = m_ExeRegion.ye;
                    m_ExeRegion.ye = dblTemp;
                }
                //処理範囲を適用
                axFvxBinFilter1.SetInspectArea(m_ExeRegion);
                axFvxDisplayEx1.ClearOverlay();
                axFvxDisplayEx1.PenColor = Color.Lime;
                axFvxDisplayEx1.DrawData(0, m_ExeRegion);
            }
        }

        //画像ファイル保存
        private void btnSaveImage_Click(object sender, EventArgs e)
        {
            saveFileDialog1.InitialDirectory = m_strDirectory;
            saveFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
            saveFileDialog1.DefaultExt = "*.bmp";
            saveFileDialog1.ShowDialog();

            if (saveFileDialog1.FileName.Length > 0)
            {
                //FileAccessコンポーネントを使って
                //Imageをファイルに保存します｡
                axFvxFileAccess1.FilePath = saveFileDialog1.FileName;
                try
                {
                    axFvxFileAccess1.SaveBitmap(axFvxImageOrg.GetOcx());
                }
                catch (Exception)
                {
                    int IntErr = 0;
                    axFvxFileAccess1.GetErrReport(out IntErr);
                    MessageBox.Show("保存に失敗しました。　ErrCode= " + IntErr.ToString());
                    return;
                }
            }

        }

        //2値レベル変更時
        private void txtBinLevel_TextChanged(object sender, EventArgs e)
        {
			if (txtBinLevel.Text.Length > 0)
			{
				m_BinLevel = int.Parse(txtBinLevel.Text);
				if (m_BinLevel > 255)
				{
					m_BinLevel = 255;
					txtBinLevel.Text = "255";
				}
				axFvxChangeImage1.FixedLevel(axFvxImageOrg.GetOcx(), axFvxImageOrgBin.GetOcx(), m_BinLevel);
				axFvxDisplayEx1.DrawImage(axFvxImageOrgBin.GetOcx());
			}
        }

        //-----------------------------------------------------------------------------
        //      フィルタ処理
        //-----------------------------------------------------------------------------

        //膨張
        private void btnExpand_Click(object sender, EventArgs e)
        {
            //処理部分が画像の一部の場合に、処理部分以外の元画像が保存されるようコピーします。
            axFvxImageOrgBin.CopyTo(axFvxImageDstBin.GetOcx());
            
            //フィルタ実行
            axFvxBinFilter1.Expand(axFvxImageOrgBin.GetOcx(), axFvxImageDstBin.GetOcx());
            
            //フィルタ処理後に続けて別の処理ができるように、元のイメージにコピーしています。
            //但し、このコピーを行うと画像の有効範囲がリセットされてしまいます。
            axFvxImageDstBin.CopyTo(axFvxImageOrgBin.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImageDstBin.GetOcx());
        }

        //収縮
        private void btnContrct_Click(object sender, EventArgs e)
        {
            axFvxImageOrgBin.CopyTo(axFvxImageDstBin.GetOcx());
            axFvxBinFilter1.Contrct(axFvxImageOrgBin.GetOcx(), axFvxImageDstBin.GetOcx());
            axFvxImageDstBin.CopyTo(axFvxImageOrgBin.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImageDstBin.GetOcx());
        }

        //4近傍膨張
        private void btnExpand4_Click(object sender, EventArgs e)
        {
            axFvxImageOrgBin.CopyTo(axFvxImageDstBin.GetOcx());
            axFvxBinFilter1.Expand4(axFvxImageOrgBin.GetOcx(), axFvxImageDstBin.GetOcx());
            axFvxImageDstBin.CopyTo(axFvxImageOrgBin.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImageDstBin.GetOcx());

        }

        //4近傍収縮
        private void btnContrct4_Click(object sender, EventArgs e)
        {
            axFvxImageOrgBin.CopyTo(axFvxImageDstBin.GetOcx());
            axFvxBinFilter1.Contrct4(axFvxImageOrgBin.GetOcx(), axFvxImageDstBin.GetOcx());
            axFvxImageDstBin.CopyTo(axFvxImageOrgBin.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImageDstBin.GetOcx());

        }

        //孤立点除去
        private void btnSolodel_Click(object sender, EventArgs e)
        {
            axFvxImageOrgBin.CopyTo(axFvxImageDstBin.GetOcx());
            axFvxBinFilter1.Solodel(axFvxImageOrgBin.GetOcx(), axFvxImageDstBin.GetOcx());
            axFvxImageDstBin.CopyTo(axFvxImageOrgBin.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImageDstBin.GetOcx());

        }

        //反転
        private void btnNot_Click(object sender, EventArgs e)
        {
            axFvxImageOrgBin.CopyTo(axFvxImageDstBin.GetOcx());
            axFvxBinFilter1.Not(axFvxImageOrgBin.GetOcx(), axFvxImageDstBin.GetOcx());
            axFvxImageDstBin.CopyTo(axFvxImageOrgBin.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImageDstBin.GetOcx());
        }

        //方向指定つき近傍膨張
        private void btnExpandDir_Click(object sender, EventArgs e)
        {
            axFvxImageOrgBin.CopyTo(axFvxImageDstBin.GetOcx());
            axFvxBinFilter1.ExpandDir(axFvxImageOrgBin.GetOcx(),axFvxImageDstBin.GetOcx(),(FVXCOMBINFILTERLib.DIRECTION_BIN_TYPE)cmbDirection1.SelectedIndex);
            axFvxImageDstBin.CopyTo(axFvxImageOrgBin.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImageDstBin.GetOcx());   
        }

        //方向指定付き近傍収縮
        private void btnContrctDir_Click(object sender, EventArgs e)
        {
            axFvxImageOrgBin.CopyTo(axFvxImageDstBin.GetOcx());
            axFvxBinFilter1.ContrctDir(axFvxImageOrgBin.GetOcx(), axFvxImageDstBin.GetOcx(),(FVXCOMBINFILTERLib.DIRECTION_BIN_TYPE)cmbDirection1.SelectedIndex);
            axFvxImageDstBin.CopyTo(axFvxImageOrgBin.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImageDstBin.GetOcx());
        }

        //ノイズ除去
        private void btnBfilter_Click(object sender, EventArgs e)
        {
            short Sx = Convert.ToInt16(txtHPix.Text);
            short Sy = Convert.ToInt16(txtVPix.Text);;

            if(Sx < 0 || 50 < Sx)
            {
                MessageBox.Show("水平除去画素数は 0～50 の範囲で入力してください", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (Sy < 0 || 50 < Sy)
            {
                MessageBox.Show("垂直除去画素数は 0～50 の範囲で入力してください", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            axFvxImageOrgBin.CopyTo(axFvxImageDstBin.GetOcx());
            axFvxBinFilter1.Bfilter(axFvxImageOrgBin.GetOcx(), axFvxImageDstBin.GetOcx(),Sx,Sy, (FVXCOMBINFILTERLib.BFILTER_TYPE)cmbDirection2.SelectedIndex);
            axFvxImageDstBin.CopyTo(axFvxImageOrgBin.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImageDstBin.GetOcx());
        }

        //エディットコントロールに数字以外が入力されたときにキャンセルする 
        private void txtBinLevel_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
                e.Handled = true;
        }
        private void txtVPix_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
                e.Handled = true;
        }
        private void txtHPix_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
                e.Handled = true;
        }
    }
}